#!/usr/bin/env python
from numpy import *
from math import *
from scipy import interpolate
import scipy
import os
import getopt
import sys
import csv
from pylab import *
from mpl_toolkits.mplot3d import Axes3D

def usage(error=1):
    usage = """\

Usage: %s [INPUT_FILE_NAME][INPUT_FILE_NAME2]

Quasi-steady model calculations. Takes a file that has format 

RShoulder_X, RShoulder_Y, RShoulder_Z, RWing_X, RWing_Y, RWing_Z, LShoulder_X, LShoulder_Y, LShoulder_Z, LWing_X, LWing_Y, LWing_Z, COM_X, COM_Y, COM_Z, R_Attack, L_Attack 

and pronation-supination_new file used for 3_wingkinematics_arc

The coordinates for the wings and shoulders must be transformed into bird frame of reference. The coordinates of the COM must be global, in order to account for flight speed. Returns a file with instantaneous aerodynamic forces and a file with forces averaged per wingstroke.

Options:
   -h, --help    Print this message

[written by Paolo Segre, 2016]


""" % os.path.basename(sys.argv[0])

    if error:
        sys.stderr.write(usage)
        sys.exit(1)
    else:
        sys.stdout.write(usage)
        sys.exit()


def mag_vector(vec):
    mag=sqrt(sum(vec**2,axis=1))
    return(mag)

def unit_vector(vec):
    unit=vec/vstack((sqrt(sum(vec**2,axis=1)),sqrt(sum(vec**2,axis=1)),sqrt(sum(vec**2,axis=1)))).T
    return(unit)


def main(file,file2):

        # input files
        (PATH, FILENAME) = os.path.split(file)
        inFile = open(file,"r")
        lines = inFile.readlines()    

        Data= zeros(((len(lines)-1),17), float)

        for N in range(1,len(lines)):
            
            temp = str(lines[N]).split(',')            

            Data[N-1,:]=(float(temp[0]),float(temp[1]),float(temp[2]),float(temp[3]),float(temp[4]),float(temp[5]),float(temp[6]),float(temp[7]),float(temp[8]),float(temp[9]),float(temp[10]),float(temp[11]),float(temp[12]),float(temp[13]),float(temp[14]),float(temp[15]),float(temp[16]))

        ###############################
        (PATH2, FILENAME2) = os.path.split(file2)
        inFile = open(file2,"r")
        lines2 = inFile.readlines()    

        temp2= str(lines2[1]).split(',')
        U=[]

        for N in range(0,len(temp2)):    
            U.append(int(temp2[N]))  

        updowns= zeros(((len(lines)-1),1), float)

        for N in range(0,len(U[1::2])):
            updowns[U[::2][N]-1:U[1::2][N]-1]=1 

        seterr(all="ignore")        

        ####
        RShoulder=Data[:,0:3]
        RWing=Data[:,3:6]
        LShoulder=Data[:,6:9]
        LWing=Data[:,9:12]

        COM=Data[:,12:15]  
      
        RAttack=Data[:,15]
        LAttack=Data[:,16]
    
        RLeadingE=RWing-RShoulder
        LLeadingE=LWing-LShoulder

###################################
        #bird parameters         
        Amp=277      #Per bird mean wingstroke amplitude
        density=1.18 #Air density kg/m^3

        mass=.00456  # SI Table S1
        winglength=.0536 # SI Table S1
        SA=762.5/(1000**2) #m^2 SI Table S1
        WingMoment2= .506 # SI Table S1
        WingMoment3= .558 # SI Table S1
        induced_vel_angle = 0 #degrees, 0 is vertical, 90 is horizontal forwards

        remove_udspikes='yes'
        framerate=10000
        units=1000 #1000 for mm, 100 for cm, 1 for m
        
        multiplier=framerate/units #used to convert from mm/frame to m/s


##############################

        ###induced velocity            
        AdiskArea=(winglength**2)*radians(Amp)
        Vi=sqrt((mass*9.81)/(2*density*AdiskArea))
        Vi_frame=Vi/multiplier #convert from m/s to mm/frame
        Vi_frame_vec= [(sin(radians(induced_vel_angle))*Vi_frame),0,(cos(radians(induced_vel_angle))*Vi_frame)]                

        ###body_vel 
        Vbody=COM[1:,:]-COM[:-1,:]        
        Vi_Vbody_frame_vec=Vi_frame_vec+Vbody     

################################

        #wing velocity
        RWingVel=RWing[1:,:]-RWing[:-1,:]
        RWingVel=RWingVel+Vi_Vbody_frame_vec
        RWingVelMag=mag_vector(RWingVel)*multiplier
        RWingVel_unit=unit_vector(RWingVel)

        LWingVel=LWing[1:,:]-LWing[:-1,:]       
        LWingVel=LWingVel+Vi_Vbody_frame_vec    
        LWingVelMag=mag_vector(LWingVel)*multiplier
        LWingVel_unit=unit_vector(LWingVel)

        RAttack2=zeros_like(RAttack,float)
        LAttack2=zeros_like(LAttack,float)
        RCL=zeros_like(RAttack,float)
        RCD=zeros_like(RAttack,float)
        LCL=zeros_like(RAttack,float)
        LCD=zeros_like(RAttack,float)

        #calculate angle of velocity
        RWingVel_horizontal=copy(RWingVel_unit) 
        RWingVel_horizontal[:,2]=0                
        Rangle=degrees(arccos((RWingVel_horizontal*RWingVel_unit).sum(axis=1)/(mag_vector(RWingVel_horizontal)*mag_vector(RWingVel_unit))))*sign(RWingVel_unit[:,2]) 

        LWingVel_horizontal=copy(LWingVel_unit) 
        LWingVel_horizontal[:,2]=0                
        Langle=degrees(arccos((LWingVel_horizontal*LWingVel_unit).sum(axis=1)/(mag_vector(LWingVel_horizontal)*mag_vector(LWingVel_unit))))*sign(LWingVel_unit[:,2]) 
     

        ######
        #adjust velocity direction for upstroke/downstroke

        for n in range(0,len(updowns)-1):
            if int(updowns[n])==1:
                RAttack2[n]=RAttack[n]-Rangle[n]
            else:
                RAttack2[n]=RAttack[n]+Rangle[n]-180

        for n in range(0,len(updowns)-1):
            if int(updowns[n])==1:
                LAttack2[n]=LAttack[n]-Langle[n]
            else:
                LAttack2[n]=LAttack[n]+Langle[n]-180


        #######################################
        #remove spikes at upstroke-downstroke caused by frame of reference rotations
        if remove_udspikes=='yes':
            ud_boundaries=where(updowns[1:]-updowns[:-1]!=0)[0]
            for n in ud_boundaries:
                RWingVelMag[n]=(RWingVelMag[n-1]+RWingVelMag[n+1])/2            
                LWingVelMag[n]=(LWingVelMag[n-1]+LWingVelMag[n+1])/2            
                RAttack2[n]=(RAttack2[n-1]+RAttack2[n+1])/2    
                LAttack2[n]=(LAttack2[n-1]+LAttack2[n+1])/2    
        ##################################################
        #calculates CL and CD based on Kruyt et. al. 2014

        for n in range(0,len(RAttack2)):
            if RAttack2[n] < 0:
                RCL[n]=.0028+1.1251*cos((0.0332*RAttack2[n])+4.6963)
                RCD[n]=1.1993+1.0938*cos((0.0281*RAttack2[n])+3.1277)
            if RAttack2[n] >= 0:
                RCL[n]= 0.0031 + 1.5842 * cos((0.0301*RAttack2[n])+4.7124)      
                RCD[n]= 8.3171 + 8.1909 * cos((0.0073*RAttack2[n])+3.1416)

        for n in range(0,len(LAttack2)):
            if LAttack2[n] < 0:
                LCL[n]=.0028+1.1251*cos((0.0332*LAttack2[n])+4.6963)
                LCD[n]=1.1993+1.0938*cos((0.0281*LAttack2[n])+3.1277)
            if LAttack2[n] >= 0:
                LCL[n]= 0.0031 + 1.5842 * cos((0.0301*LAttack2[n])+4.7124)      
                LCD[n]= 8.3171 + 8.1909 * cos((0.0073*LAttack2[n])+3.1416)
        ##########################
        #calculates magnitude and direction of lift and drag
   
        RLiftDirection=unit_vector(cross(RLeadingE[:-1],RWingVel))
        LLiftDirection=-unit_vector(cross(LLeadingE[:-1],LWingVel))         
        RLift_magnitude=.5*(RCL[:-1]*density*SA*(WingMoment2*RWingVelMag)**2)
        LLift_magnitude=.5*(LCL[:-1]*density*SA*(WingMoment2*LWingVelMag)**2)

        RDragDirection=-RWingVel_unit
        LDragDirection=-LWingVel_unit

        RDrag_magnitude=.5*(RCD[:-1]*density*SA*(WingMoment3*RWingVelMag)**2)*WingMoment3
        LDrag_magnitude=.5*(LCD[:-1]*density*SA*(WingMoment3*LWingVelMag)**2)*WingMoment3

        #vectorized lift and drag
         
        RLift=array([RLift_magnitude]).T*RLiftDirection
        LLift=array([LLift_magnitude]).T*LLiftDirection

        RDrag=array([RDrag_magnitude]).T*RDragDirection
        LDrag=array([LDrag_magnitude]).T*LDragDirection

        RLift_vector=zeros_like(RLift,float)        
        LLift_vector=zeros_like(LLift,float) 
        
        #in terms of %bodymass        
        RLift_vector=(RLift+RDrag)*(100/(mass*9.81))
        LLift_vector=(LLift+LDrag)*(100/(mass*9.81))

        total_force=RLift_vector+LLift_vector
        average_force=average(total_force[U[0]-1:U[-1]-1,:],axis=0)

        print 'avg body vel', average(mag_vector(Vbody[U[0]-1:U[-1]-1]))*multiplier 
        print 'induced vel', Vi
        print 'avg vel', average(RWingVelMag[U[0]-1:U[-1]-1])
        print 'force', average_force 

    #####output         

        Writer = csv.writer(open(PATH2+'/aero/aero_analysis_'+FILENAME[:-4]+'.csv', 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)       
            
        Writer.writerow(("trial","wingbeat","force_ds_x","force_ds_y","force_ds_z","force_us_x","force_us_y","force_us_z","R_force_ds_x","R_force_ds_y","R_force_ds_z","R_force_us_x","R_force_us_y","R_force_us_z","L_force_ds_x","L_force_ds_y","L_force_ds_z","L_force_us_x","L_force_us_y","L_force_us_z"))
        
        for N in range(0,len(U[1::2])):
            ds_start=U[::2][N]-1
            ds_stop=U[1::2][N]-1
            us_start=U[1::2][N]-1
            us_stop=U[2::2][N]-1

            Writer.writerow(hstack((FILENAME2[:-10],N+1,average(total_force[ds_start:ds_stop],axis=0),average(total_force[us_start:us_stop],axis=0),average(RLift_vector[ds_start:ds_stop],axis=0),average(RLift_vector[us_start:us_stop],axis=0),average(LLift_vector[ds_start:ds_stop],axis=0),average(LLift_vector[us_start:us_stop],axis=0))))

            #print average(total_force[ds_start:ds_stop],axis=0)
            #print average(total_force[us_start:us_stop],axis=0)


        ##########################
        ######plots

        if 0:

            a_unit=RLift*500
            b_unit=LLift*500
    
            c_unit=RDrag*500
            d_unit=LDrag*500


            fig = plt.figure()
            ax = fig.add_subplot(111,projection='3d')
            
            for n in range(0,len(RWing[:,0])-1,5):

                plot([RShoulder[n,0],RWing[n,0]],[RShoulder[n,1],RWing[n,1]],[RShoulder[n,2],RWing[n,2]],c='k')
                plot([LShoulder[n,0],LWing[n,0]],[LShoulder[n,1],LWing[n,1]],[LShoulder[n,2],LWing[n,2]],c='k')    

                ax.scatter(RWing[n,0],RWing[n,1],RWing[n,2],c='b') 
                ax.scatter(LWing[n,0],LWing[n,1],LWing[n,2],c='r') 

                plot([RWing[n,0],RWing[n,0]+a_unit[n,0]],[RWing[n,1],RWing[n,1]+a_unit[n,1]],[RWing[n,2],RWing[n,2]+a_unit[n,2]],c='g')   
         
                plot([RWing[n,0],RWing[n,0]+c_unit[n,0]],[RWing[n,1],RWing[n,1]+c_unit[n,1]],[RWing[n,2],RWing[n,2]+c_unit[n,2]],c='c')   

                plot([LWing[n,0],LWing[n,0]+b_unit[n,0]],[LWing[n,1],LWing[n,1]+b_unit[n,1]],[LWing[n,2],LWing[n,2]+b_unit[n,2]],c='g')   
         
                plot([LWing[n,0],LWing[n,0]+d_unit[n,0]],[LWing[n,1],LWing[n,1]+d_unit[n,1]],[LWing[n,2],LWing[n,2]+d_unit[n,2]],c='c')   

            ax.auto_scale_xyz([-50, 50], [-50, 50], [20, 120])
        
            show()

        if 0:
            clf()
            figure(2)
            plot(LAttack2)
            plot(RAttack2) 
            show()

        if 0:
            clf()
            plot(RCL)
            plot(RCD)
            plot(LCL)
            plot(LCD)
            show()

        if 0:
            clf()
            plot(RWingVelMag)
            plot(LWingVelMag)
            plot(RAttack)
            show()   

        if 0:
            clf()
            x=arange(-90,90)

            print x
            y=zeros_like(x,float)
            z=zeros_like(x,float)
            for n in range(0,len(x)):
                if x[n] < 0:
                    y[n]=.0028+1.1251*cos((0.0332*x[n])+4.6963)
                    z[n]=1.1993+1.0938*cos((0.0281*x[n])+3.1277)
                if x[n] >= 0:
                    y[n]= 0.0031 + 1.5842 * cos((0.0301*x[n])+4.7124)      
                    z[n]= 8.3171 + 8.1909 * cos((0.0073*x[n])+3.1416)
                print x[n],y[n], 0.0031 + 1.5842 * cos((0.0301*x[n])+4.7124)
    
            plot(x,y)
            plot(x,z)
            show()


        if 0:
            clf()
            plot(RLift_magnitude)
            plot(RDrag_magnitude)
            plot(LLift_magnitude)
            plot(LDrag_magnitude)
            show()


        if 1:            

            start=U[0]-1
            stop=U[-1]-1

            upstroke_downstroke=(updowns*500-250)[start:stop-1,0]
            x=arange(0,len(upstroke_downstroke))

            clf()
            subplot(3,1,1)
            plot(RLift_vector[start:stop,2],c='b')
            plot(LLift_vector[start:stop,2],c='r')
            plot(total_force[start:stop,2],c='k')
            fill_between(x,-250,upstroke_downstroke,color='grey')
            axhline(average_force[2],c='g')
            ylim(-50,250)
            xlim(0,len(lines))
    
            subplot(3,1,2)
            plot(RLift_vector[start:stop,0],c='b')
            plot(LLift_vector[start:stop,0],c='r')
            plot(total_force[start:stop,0],c='k')
            fill_between(x,-250,upstroke_downstroke,color='grey')
            axhline(average_force[0],c='g')
            ylim(-150,150)
            xlim(0,len(lines))
    
            subplot(3,1,3)
            plot(RLift_vector[start:stop,1],c='b')
            plot(LLift_vector[start:stop,1],c='r')
            plot(total_force[start:stop,1],c='k')
            fill_between(x,-250,upstroke_downstroke,color='grey')
            axhline(average_force[1],c='g')
            ylim(-150,150)
            xlim(0,len(lines))

            savefig(PATH2+'/aero/'+FILENAME[:-4]+'.png')
            #show() 

######################################################################################################################################################
if __name__ == '__main__':

        try:
            opts, args = getopt.getopt(sys.argv[1:], 'h', ['help'])
    

        except getopt.error:
            sys.stderr.write("%s: %s \nTry `%s --help` for more informatIon\n"
                % (PROGNAME, e, PROGNAME))
            sys.exit(1)


        for o in opts:
            if o in ('-h', '--help'):
                usage(error=0)

        if len(args) != 2:
            usage()

        main(args[0],args[1])
